import requests
from .auth import MpesaBase


class TransactionStatus(MpesaBase):
    def __init__(self, env="sandbox", app_key=None, app_secret=None, sandbox_url=None, live_url=None):
        MpesaBase.__init__(self, env, app_key, app_secret,
                           sandbox_url, live_url)
        self.authentication_token = self.authenticate()

    def check_transaction_status(self, party_a=None, identifier_type=None, remarks=None, initiator=None, security_credential=None,
                                 result_url=None, queue_timeout_url=None, transaction_id=None,
                                 occassion=None):
        """This method uses Mpesa's Transaction Status API to check the status of a transaction.

            **Args:**
                - party_a (str): Organization/MSISDN receiving the transaction - MSISDN or shortcode.
                - identifier_type (str): Type of organization receiving the transaction
                 1-MSISDN. 2-Till Number, 4-Shortcode.
                - remarks (str): Comments that are sent along with the transaction(maximum 100 characters).
                - initiator (str): This is the credential/username used to authenticate the transaction request.
                - security_credential (str): Get from developer portal
                - result_url (str): The url that handles information from the mpesa API call.
                - transaction_id (str): Unique identifier to identify a transaction on M-Pesa.
                - queue_timeout_url (str): The url that stores information of timed out transactions.
                - result_url (str): The url that receives results from M-Pesa api call.
                - occassion (str):


            **Returns:**
                - ResultDesc': ,
                - CheckoutRequestID': ,
                - ResponseDescription': ,
                - MerchantRequestID': ,
                - ResponseCode': ,
                - ResultCode':


        """

        payload = {
            "CommandID": "TransactionStatusQuery",
            "PartyA": party_a,
            "IdentifierType": identifier_type,
            "Remarks": remarks,
            "Initiator": initiator,
            "SecurityCredential": security_credential,
            "QueueTimeOutURL": queue_timeout_url,
            "ResultURL": result_url,
            "TransactionID": transaction_id,
            "Occasion": occassion
        }
        headers = {'Authorization': f"Bearer {self.authentication_token}", 'Content-Type': "application/json"}
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        saf_url = f"{base_safaricom_url}/mpesa/transactionstatus/v1/query"
        r = requests.post(saf_url, headers=headers, json=payload)
        return r.json()



import pytest
from .base import mp


@pytest.mark.usefixtures("mock_fixture_test_auth")
def test_auth(auth):
    auth = mp.C2B.authenticate()
    assert auth is not None


@pytest.mark.usefixtures("mock_fixture_test_b2c")
def test_b2c(b2c):
    data = {"initiator_name": "testapi364",
            "security_credential": "TziD/ydlT52Fm6SOH1ebrzUFwy3cP6OGplsrWja+X/1roQy2AzMsj5QGuqu9O+IFR1E6l16Jm87tg4bhnxoIhAufCEWusQI1wJZ6YLzpN0cHZAY/8SN1JfHdgEkrmksAY14pejHyfntyLT9Sg51kBjaj6J7/2+gHl2e64klnJAhlfPJWxC18zwEzsg58zFmypcovPPB6MHkPLyHQNFbu4oXC0e2gkZrIAWXTNN7PpYt4m/w39s5txU7/6P7hTzXgYAgqk4kxfPBIBeEmKhH5tSGxMD+xnSpZIXLovFgopexq8S76pmdLMjr2CdR60GlwXnAPnKJ5U9CIxRRewuoksQ==",
            "amount": "1000",
            "command_id": "SalaryPayment",
            "party_a": "600977",
            "party_b": "254708374149",
            "remarks": "Just a Transaction",
            "queue_timeout_url": "https://testurl.com/b2b-timeout",
            "result_url": "https://testurl.com/b2b-result",
            "occassion": "Test123"
            }
    resp = mp.B2C.transact(**data)
    assert dict(resp).keys() == b2c.keys()


@pytest.mark.usefixtures("mock_fixture_test_express_sim")
def test_express_sim(express_sim):
    data = {
        "business_shortcode": 174379,
        "passcode": "bfb279f9aa9bdbcf158e97dd71a467cd2e0c893059b10f78e6b72ada1ed2c919",
        "amount": "1",
        "phone_number": "254713812939",
        "callback_url": "https://testurl.com/b2b-result",
        "reference_code": "DSC",
        "description": "first test"
    }
    resp = mp.MpesaExpress.stk_push(**data)
    assert dict(resp).keys() == express_sim.keys()


@pytest.mark.usefixtures("mock_fixture_test_express_query")
def test_express_query(express_query):
    data = {
        "business_shortcode": 174379,
        "passcode": "bfb279f9aa9bdbcf158e97dd71a467cd2e0c893059b10f78e6b72ada1ed2c919",
        "checkout_request_id": "ws_CO_021020210904344355"
    }
    resp = mp.MpesaExpress.query(**data)
    assert dict(resp).keys() == express_query.keys()


@pytest.mark.usefixtures("mock_fixture_test_c2b_reg")
def test_c2b_reg(c2b_reg):
    data = {"shortcode": "600364",
            "response_type": "Completed",
            "confirmation_url": "https://testurl.com/confirmation",
            "validation_url": "https://testurl.com/validation"
            }
    resp = mp.C2B.register(**data)
    assert dict(resp).keys() == c2b_reg.keys()


@pytest.mark.usefixtures("mock_fixture_test_c2b_sim")
def test_c2b_sim(c2b_sim):
    data = {"shortcode": "600364",
            "command_id": "CustomerPayBillOnline",
            "amount": "123",
            "msisdn": "254708374149",
            "bill_ref_number": "account"
            }
    resp = mp.C2B.simulate(**data)
    assert dict(resp).keys() == c2b_sim.keys()


@pytest.mark.usefixtures("mock_fixture_test_bal")
def test_bal(bal):
    data = {"initiator": "testapi364",
            "security_credential": "TziD/ydlT52Fm6SOH1ebrzUFwy3cP6OGplsrWja+X/1roQy2AzMsj5QGuqu9O+IFR1E6l16Jm87tg4bhnxoIhAufCEWusQI1wJZ6YLzpN0cHZAY/8SN1JfHdgEkrmksAY14pejHyfntyLT9Sg51kBjaj6J7/2+gHl2e64klnJAhlfPJWxC18zwEzsg58zFmypcovPPB6MHkPLyHQNFbu4oXC0e2gkZrIAWXTNN7PpYt4m/w39s5txU7/6P7hTzXgYAgqk4kxfPBIBeEmKhH5tSGxMD+xnSpZIXLovFgopexq8S76pmdLMjr2CdR60GlwXnAPnKJ5U9CIxRRewuoksQ==",
            "command_id": "AccountBalance",
            "party_a": "600364",
            "identifier_type": "4",
            "remarks": "Just a Transaction",
            "queue_timeout_url": "https://testurl.com/b2b-timeout",
            "result_url": "https://testurl.com/b2b-result"
            }
    resp = mp.Balance.get_balance(**data)
    assert dict(resp).keys() == bal.keys()


@pytest.mark.usefixtures("mock_fixture_test_t_status")
def test_t_status(t_status):
    data = {"initiator": "testapi364",
            "transaction_id": "PJ251HK6YH",
            "party_a": "600364",
            "security_credential": "TziD/ydlT52Fm6SOH1ebrzUFwy3cP6OGplsrWja+X/1roQy2AzMsj5QGuqu9O+IFR1E6l16Jm87tg4bhnxoIhAufCEWusQI1wJZ6YLzpN0cHZAY/8SN1JfHdgEkrmksAY14pejHyfntyLT9Sg51kBjaj6J7/2+gHl2e64klnJAhlfPJWxC18zwEzsg58zFmypcovPPB6MHkPLyHQNFbu4oXC0e2gkZrIAWXTNN7PpYt4m/w39s5txU7/6P7hTzXgYAgqk4kxfPBIBeEmKhH5tSGxMD+xnSpZIXLovFgopexq8S76pmdLMjr2CdR60GlwXnAPnKJ5U9CIxRRewuoksQ==",
            "identifier_type": "4",
            "remarks": "Enterance Fee",
            "queue_timeout_url": "https://3e61-197-156-137-143.ngrok.io/b2b-timeout",
            "result_url": "https://3e61-197-156-137-143.ngrok.io/b2b-result",
            "occassion": "DSC-party_a"
            }
    resp = mp.TransactionStatus.check_transaction_status(**data)
    assert dict(resp).keys() == t_status.keys()


@pytest.mark.usefixtures("mock_fixture_test_b2b")
def test_b2b(b2b):
    data = {"initiator": "testapi364",
            "security_credential": "TziD/ydlT52Fm6SOH1ebrzUFwy3cP6OGplsrWja+X/1roQy2AzMsj5QGuqu9O+IFR1E6l16Jm87tg4bhnxoIhAufCEWusQI1wJZ6YLzpN0cHZAY/8SN1JfHdgEkrmksAY14pejHyfntyLT9Sg51kBjaj6J7/2+gHl2e64klnJAhlfPJWxC18zwEzsg58zFmypcovPPB6MHkPLyHQNFbu4oXC0e2gkZrIAWXTNN7PpYt4m/w39s5txU7/6P7hTzXgYAgqk4kxfPBIBeEmKhH5tSGxMD+xnSpZIXLovFgopexq8S76pmdLMjr2CdR60GlwXnAPnKJ5U9CIxRRewuoksQ==",
            "amount": "100",
            "command_id": "BusinessPayBill",
            "sender_identifier_type": "4",
            "receiver_identifier_type": "4",
            "party_a": "600364",
            "party_b": "600000",
            "remarks": "Enterance Fee",
            "queue_timeout_url": "https://4ca0-197-156-137-168.ngrok.io/b2b-timeout",
            "result_url": "https://4ca0-197-156-137-168.ngrok.io/b2b-result",
            "account_reference": "DSC-party_a"
            }
    resp = mp.B2B.transact(**data)
    assert dict(resp).keys() == b2b.keys()



AUTH = {
    "access_token": "1A7zSU7kG5cCTIfAY1i32oUvW0Z5",
    "expires_in": "3599"
}
B2C = {
    "ConversationID": "AG_20210929_00004d202f0f95df3b88",
    "OriginatorConversationID": "62947-35666607-1",
    "ResponseCode": "0",
    "ResponseDescription": "Accept the service request successfully."
}
EXPRESS_SIM = {
    "MerchantRequestID": "25811-34836046-1",
    "CheckoutRequestID": "ws_CO_290920211026123753",
    "ResponseCode": "0",
    "ResponseDescription": "Success. Request accepted for processing",
    "CustomerMessage": "Success. Request accepted for processing"
}
EXPRESS_QUERY = {
    "ResponseCode": "0",
    "ResponseDescription": "The service request has been accepted successsfully",
    "MerchantRequestID": "25811-34836046-1",
    "CheckoutRequestID": "ws_CO_290920211026123753",
    "ResultCode": "1032",
    "ResultDesc": "Request cancelled by user"
}
C2B_REG = {
    "OriginatorCoversationID": "31325-15385456-1",
    "ResponseCode": "0",
    "ResponseDescription": "success"
}

C2B_SIM = {
    "OriginatorCoversationID": "54609-35532657-1",
    "ResponseCode": "0",
    "ResponseDescription": "Accept the service request successfully."
}

BAL = {
    "OriginatorConversationID": "18795-24511603-1",
    "ConversationID": "AG_20210929_00005b98cef717428741",
    "ResponseCode": "0",
    "ResponseDescription": "Accept the service request successfully."
}

TRANSACTION_STATUS = {
    "OriginatorConversationID": "117879-42739630-1",
    "ConversationID": "AG_20211002_0000635cc3241d2055a8",
    "ResponseCode": "0",
    "ResponseDescription": "Accept the service request successfully."
}
B2B = {
    'OriginatorConversationID': '77333-45263982-1',
    'ConversationID': 'AG_20211003_00004e5173b463fe1447',
    'ResponseCode': '0',
    'ResponseDescription': 'Accept the service request successfully.'
}



import base64
import requests
from .auth import MpesaBase
import datetime


class MpesaExpress(MpesaBase):
    def __init__(self, env="sandbox", app_key=None, app_secret=None, sandbox_url=None, live_url=None):
        MpesaBase.__init__(self, env, app_key, app_secret,
                           sandbox_url, live_url)
        self.authentication_token = self.authenticate()

    def stk_push(self, business_shortcode=None, passcode=None, amount=None, callback_url=None, reference_code=None,
                 phone_number=None, description=None):
        """This method uses Mpesa's Express API to initiate online payment on behalf of a customer..

        **Args:**
            - business_shortcode (int): The short code of the organization.
            - passcode (str): Get from developer portal
            - amount (int): The amount being transacted
            - callback_url (str): A CallBack URL is a valid secure URL that is used to receive notifications from M-Pesa API.
            - reference_code: Account Reference: This is an Alpha-Numeric parameter that is defined by your system
                as an Identifier of the transaction for CustomerPayBillOnline transaction type.
            - phone_number: The Mobile Number to receive the STK Pin Prompt.
            - description: This is any additional information/comment that can be sent along with
                the request from your system. MAX 13 characters


        **Returns:**
            - CustomerMessage (str):
            - CheckoutRequestID (str):
            - ResponseDescription (str):
            - MerchantRequestID (str):
            - ResponseCode (str):

        """

        time = str(datetime.datetime.now()).split(".")[0].replace(
            "-", "").replace(" ", "").replace(":", "")
        password = f"{business_shortcode}{passcode}{time}"
        encoded = base64.b64encode(password.encode())
        payload = {
            "BusinessShortCode": business_shortcode,
            "Password": encoded.decode('utf-8'),
            "Timestamp": time,
            "TransactionType": "CustomerPayBillOnline",
            "Amount": amount,
            "PartyA": int(phone_number),
            "PartyB": business_shortcode,
            "PhoneNumber": int(phone_number),
            "CallBackURL": callback_url,
            "AccountReference": reference_code,
            "TransactionDesc": description
        }
        headers = {'Authorization': f"Bearer {self.authentication_token}", 'Content-Type': "application/json"}
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        saf_url = f"{base_safaricom_url}/mpesa/stkpush/v1/processrequest"
        r = requests.post(saf_url, headers=headers, json=payload)
        return r.json()

    def query(self, business_shortcode=None, checkout_request_id=None, passcode=None):
        """This method uses Mpesa's Express API to check the status of a Lipa Na M-Pesa Online Payment..

                                                    **Args:**
                                                        - business_shortcode (int): This is organizations shortcode (Paybill or Buygoods - A 5 to 6 digit account number) used to identify an organization and receive the transaction.
                                                        - checkout_request_id (str): This is a global unique identifier of the processed checkout transaction request.
                                                        - passcode (str): Get from developer portal


                                                    **Returns:**
                                                        - CustomerMessage (str):
                                                        - CheckoutRequestID (str):
                                                        - ResponseDescription (str):
                                                        - MerchantRequestID (str):
                                                        - ResponseCode (str):


        """

        time = str(datetime.datetime.now()).split(".")[0].replace(
            "-", "").replace(" ", "").replace(":", "")
        password = f"{business_shortcode}{passcode}{time}"
        encoded = base64.b64encode(password.encode())
        payload = {
            "BusinessShortCode": business_shortcode,
            "Password": str(encoded),
            "Timestamp": time,
            "CheckoutRequestID": checkout_request_id
        }
        headers = {'Authorization': f"Bearer {self.authentication_token}", 'Content-Type': "application/json"}
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        saf_url = f"{base_safaricom_url}/mpesa/stkpushquery/v1/query"
        r = requests.post(saf_url, headers=headers, json=payload)
        return r.json()



import requests
from .auth import MpesaBase


class B2C(MpesaBase):
    def __init__(self, env="sandbox", app_key=None, app_secret=None, sandbox_url="https://sandbox.safaricom.co.ke",
                 live_url="https://safaricom.co.ke"):
        MpesaBase.__init__(self, env, app_key, app_secret,
                           sandbox_url, live_url)
        self.authentication_token = self.authenticate()

    def transact(self, initiator_name=None, security_credential=None, command_id=None, amount=None,
                 party_a=None, party_b=None, remarks=None,
                 queue_timeout_url=None, result_url=None, occassion=None):
        """This method uses Mpesa's B2C API to transact between an M-Pesa short
        code to a phone number registered on M-Pesa..

                    **Args:**
                        - initiator_name (str): Username used to authenticate the transaction.
                        - security_credential (str): Generate from developer portal
                        - command_id (str): Options: SalaryPayment, BusinessPayment, PromotionPayment
                        - amount(str): Amount.
                        - party_a (int): B2C organization shortcode from which the money is to be sent.Shortcode (5-6 digits) - MSISDN (12 digits).
                        - party_b (int): MSISDN receiving the transaction (12 digits). Should start with 254 without + sign.
                        - remarks (str): Comments that are sent along with the transaction(maximum 100 characters).
                        - account_reference (str): Use if doing paybill to banks etc.
                        - queue_timeout_url (str): The url that handles information of timed out transactions.
                        - result_url (str): The url that receives results from M-Pesa api call.
                        - ocassion (str):


                    **Returns:**
                        - OriginatorConverstionID (str): The unique request ID for tracking a transaction.
                        - ConversationID (str): The unique request ID returned by mpesa for each request made
                        - ResponseDescription (str): Response Description message


        """

        payload = {
            "InitiatorName": initiator_name,
            "SecurityCredential": security_credential,
            "CommandID": command_id,
            "Amount": amount,
            "PartyA": party_a,
            "PartyB": party_b,
            "Remarks": remarks,
            "QueueTimeOutURL": queue_timeout_url,
            "ResultURL": result_url,
            "Occassion": occassion
        }
        headers = {'Authorization': f"Bearer {self.authentication_token}", 'Content-Type': "application/json"}
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        saf_url = f"{base_safaricom_url}/mpesa/b2c/v1/paymentrequest"
        r = requests.post(saf_url, headers=headers, json=payload)
        return r.json()



from flask import Flask
from flask_mpesa import MpesaAPI
import os
app = Flask(__name__)

app.config["API_ENVIRONMENT"] = "sandbox"
app.config["APP_KEY"] = os.environ.get("MPESA_API_KEY")
app.config["APP_SECRET"] = os.environ.get("MPESA_API_SECRET")
mp = MpesaAPI(app)



from flask import Blueprint, current_app
import warnings
from flask_mpesa.api.b2c import MpesaBase
from flask_mpesa.api.b2c import B2C
from flask_mpesa.api.c2b import C2B
from flask_mpesa.api.b2b import B2B
from flask_mpesa.api.balance import Balance
from flask_mpesa.api.mpesa_express import MpesaExpress
from flask_mpesa.api.reversal import Reversal
from flask_mpesa.api.transaction_status import TransactionStatus


class MpesaAPI(object):

    def __init__(self, app=None, **kwargs):
        self.app = app
        self.sandbox_url = "https://sandbox.safaricom.co.ke"
        self.live_url = "https://safaricom.co.ke"

        if app:
            self.init_app(app)

    def init_app(self, app):
        if('APP_KEY' not in app.config and 'APP_SECRET' not in app.config and 'API_ENVIRONMENT' not in app.config):
            warnings.warn(
                'Neither APP_KEY nor APP_SECRET nor API_ENVIRONMENT is set')

        self.app = app
        self.register_blueprint(app)

    @staticmethod
    def register_blueprint(app):
        module = Blueprint('mpesaapi', __name__, template_folder='templates')
        app.register_blueprint(module)
        return module

    def get_credentials(self):
        """Helper method to return app_key, app_secret and api_environmentfrom the app's config."""
        api_env = self.app.config.get("API_ENVIRONMENT")
        app_key = self.app.config.get("APP_KEY")
        app_secret = self.app.config.get("APP_SECRET")
        return api_env, app_key, app_secret

    def get_app(self, reference_app=None):
        """Helper method that implements the logic to look up an
        application."""

        if reference_app is not None:
            return reference_app

        if current_app:
            return current_app._get_current_object()

        if self.app is not None:
            return self.app

        raise RuntimeError(
            'No application found. Either work inside a view function or push'
            ' an application context.'
        )

    @property
    def C2B(self):
        """Method returns a C2B Object"""
        env, app_key, app_secret = self.get_credentials()
        return C2B(env, app_key, app_secret, self.sandbox_url, self.live_url)

    @property
    def B2C(self):
        """Method returns a B2C Object"""
        env, app_key, app_secret = self.get_credentials()
        return B2C(env, app_key, app_secret, self.sandbox_url, self.live_url)

    @property
    def B2B(self):
        """Method returns a B2B Object"""
        env, app_key, app_secret = self.get_credentials()
        return B2B(env, app_key, app_secret, self.sandbox_url, self.live_url)

    @property
    def Balance(self):
        """Method returns a Balance Object"""
        env, app_key, app_secret = self.get_credentials()
        return Balance(env, app_key, app_secret, self.sandbox_url, self.live_url)

    @property
    def MpesaExpress(self):
        """Method returns a MpesaExpress Object"""
        env, app_key, app_secret = self.get_credentials()
        return MpesaExpress(env, app_key, app_secret, self.sandbox_url, self.live_url)

    @property
    def Reversal(self):
        """Method returns a Reversal Object"""
        env, app_key, app_secret = self.get_credentials()
        return Reversal(env, app_key, app_secret, self.sandbox_url, self.live_url)

    @property
    def TransactionStatus(self):
        """Method returns a TransactionSatus Object"""
        env, app_key, app_secret = self.get_credentials()
        return TransactionStatus(env, app_key, app_secret, self.sandbox_url, self.live_url)



import requests
from .auth import MpesaBase


class C2B(MpesaBase):
    def __init__(self, env="sandbox", app_key=None, app_secret=None, sandbox_url=None, live_url=None):
        MpesaBase.__init__(self, env, app_key, app_secret,
                           sandbox_url, live_url)
        self.authentication_token = self.authenticate()

    def register(self, shortcode=None, response_type=None, confirmation_url=None, validation_url=None):
        """This method uses Mpesa's C2B API to register validation and confirmation URLs on M-Pesa.

        **Args:**
            - shortcode (int): The short code of the organization.
            - response_type (str): Default response type for timeout. Incase a tranaction times out,
             Mpesa will by default Complete or Cancel the transaction.
            - confirmation_url (str): Confirmation URL for the client.
            - validation_url (str): Validation URL for the client.


        **Returns:**
            - OriginatorConverstionID (str): The unique request ID for tracking a transaction.
            - ConversationID (str): The unique request ID returned by mpesa for each request made
            - ResponseDescription (str): Response Description message


        """

        payload = {
            "ShortCode": shortcode,
            "ResponseType": response_type,
            "ConfirmationURL": confirmation_url,
            "ValidationURL": validation_url
        }
        headers = {'Authorization': f"Bearer {self.authentication_token}", 'Content-Type': "application/json"}
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        saf_url = f"{base_safaricom_url}/mpesa/c2b/v1/registerurl"
        r = requests.post(saf_url, headers=headers, json=payload)
        return r.json()

    def simulate(self, shortcode=None, command_id=None, amount=None, msisdn=None, bill_ref_number=None):
        """This method uses Mpesa's C2B API to simulate a C2B transaction.

        **Args:**
            - shortcode (int): The short code of the organization.
            - command_id (str): Unique command for each transaction type.
             - CustomerPayBillOnline - CustomerBuyGoodsOnline.
            - amount (int): The amount being transacted
            - msisdn (int): Phone number (msisdn) initiating the transaction MSISDN(12 digits)
            - bill_ref_number: Optional


        **Returns:**
            - OriginatorConverstionID (str): The unique request ID for tracking a transaction.
            - ConversationID (str): The unique request ID returned by mpesa for each request made
            - ResponseDescription (str): Response Description message


        """

        payload = {
            "ShortCode": shortcode,
            "CommandID": command_id,
            "Amount": amount,
            "Msisdn": msisdn,
            "BillRefNumber": bill_ref_number
        }
        headers = {'Authorization': f"Bearer {self.authentication_token}", 'Content-Type': "application/json"}
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        saf_url = f"{base_safaricom_url}/mpesa/c2b/v1/simulate"
        r = requests.post(saf_url, headers=headers, json=payload)
        return r.json()



import requests
from requests.auth import HTTPBasicAuth


class MpesaBase:
    def __init__(self, env="sandbox", app_key=None, app_secret=None, sandbox_url="https://sandbox.safaricom.co.ke",
                 live_url="https://safaricom.co.ke"):
        self.env = env
        self.app_key = app_key
        self.app_secret = app_secret
        self.sandbox_url = sandbox_url
        self.live_url = live_url
        self.token = None

    def authenticate(self):
        """To make Mpesa API calls, you will need to authenticate your app. This method is used to fetch the access token
        required by Mpesa. Mpesa supports client_credentials grant type. To authorize your API calls to Mpesa,
        you will need a Basic Auth over HTTPS authorization token. The Basic Auth string is a base64 encoded string
        of your app's client key and client secret.

            **Args:**
                - env (str): Current app environment. Options: sandbox, live.
                - app_key (str): The app key obtained from the developer portal.
                - app_secret (str): The app key obtained from the developer portal.
                - sandbox_url (str): Base Safaricom sandbox url.
                - live_url (str): Base Safaricom live url.

            **Returns:**
                - access_token (str): This token is to be used with the Bearer header for further API calls to Mpesa.

            """
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        authenticate_uri = "/oauth/v1/generate?grant_type=client_credentials"
        authenticate_url = f"{base_safaricom_url}{authenticate_uri}"
        r = requests.get(authenticate_url,
                         auth=HTTPBasicAuth(str(self.app_key), str(self.app_secret)))
        self.token = r.json()['access_token']
        return r.json()['access_token']



import requests
from .auth import MpesaBase


class B2B(MpesaBase):
    def __init__(self, env="sandbox", app_key=None, app_secret=None, sandbox_url="https://sandbox.safaricom.co.ke",
                 live_url="https://safaricom.co.ke"):
        MpesaBase.__init__(self, env, app_key, app_secret, sandbox_url, live_url)
        self.authentication_token = self.authenticate()

    def transact(self, initiator=None, security_credential=None, command_id=None, sender_identifier_type=None,
                 receiver_identifier_type=None, amount=None, party_a=None, party_b=None, remarks=None,
                 account_reference=None, queue_timeout_url=None, result_url=None):
        """This method uses Mpesa's B2B API to transact from one company to another.

            **Args:**
                - initiator (str): Username used to authenticate the transaction.
                - security_credential (str): Generate from developer portal
                - command_id (str): Options: BusinessPayBill, BusinessBuyGoods, DisburseFundsToBusiness,
                    BusinessToBusinessTransfer ,BusinessTransferFromMMFToUtility, BusinessTransferFromUtilityToMMF,
                     MerchantToMerchantTransfer, MerchantTransferFromMerchantToWorking, MerchantServicesMMFAccountTransfer,
                     AgencyFloatAdvance
                - sender_identifier_type (str): 2 for Till Number, 4 for organization shortcode
                - receiver_identifier_type (str): # 2 for Till Number, 4 for organization shortcode
                - amount(str): Amount.
                - party_a (int): Sender shortcode.
                - party_b (int): Receiver shortcode.
                - remarks (str): Comments that are sent along with the transaction(maximum 100 characters).
                - account_reference (str): Use if doing paybill to banks etc.
                - queue_timeout_url (str): The url that handles information of timed out transactions.
                - result_url (str): The url that receives results from M-Pesa api call.


            **Returns:**
                - OriginatorConverstionID (str): The unique request ID for tracking a transaction.
                - ConversationID (str): The unique request ID returned by mpesa for each request made
                - ResponseDescription (str): Response Description message


        """

        payload = {
            "Initiator": initiator,
            "SecurityCredential": security_credential,
            "CommandID": command_id,
            "SenderIdentifierType": sender_identifier_type,
            "RecieverIdentifierType": receiver_identifier_type,
            "Amount": amount,
            "PartyA": party_a,
            "PartyB": party_b,
            "Remarks": remarks,
            "AccountReference": account_reference,
            "QueueTimeOutURL": queue_timeout_url,
            "ResultURL": result_url
        }
        headers = {'Authorization': 'Bearer {0}'.format(self.authentication_token), 'Content-Type': "application/json"}
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        saf_url = "{0}{1}".format(base_safaricom_url, "/mpesa/b2b/v1/paymentrequest")
        r = requests.post(saf_url, headers=headers, json=payload)
        return r.json()



import pytest
from flask_mpesa.tests.data import AUTH, B2C, EXPRESS_SIM, EXPRESS_QUERY, C2B_REG, C2B_SIM, BAL, TRANSACTION_STATUS, B2B

base_url = "https://sandbox.safaricom.co.ke"


@pytest.fixture()
def auth():
    return AUTH


@pytest.fixture()
def b2c():
    return B2C


@pytest.fixture()
def express_sim():
    return EXPRESS_SIM


@pytest.fixture()
def express_query():
    return EXPRESS_QUERY


@pytest.fixture()
def c2b_reg():
    return C2B_REG


@pytest.fixture()
def c2b_sim():
    return C2B_SIM


@pytest.fixture()
def bal():
    return BAL


@pytest.fixture()
def t_status():
    return TRANSACTION_STATUS


@pytest.fixture()
def b2b():
    return B2B


@pytest.fixture(name="mock_fixture_test_auth")
def fixture_mock_test_auth(requests_mock):
    url = base_url + "/oauth/v1/generate?grant_type=client_credentials"
    requests_mock.get(url, json=AUTH)


@pytest.fixture(name="mock_fixture_test_b2c")
def fixture_mock_test_b2c(requests_mock, b2c):
    url = base_url + "/oauth/v1/generate?grant_type=client_credentials"
    requests_mock.get(url, json=AUTH)
    url = base_url + "/mpesa/b2c/v1/paymentrequest"
    requests_mock.post(url, json=b2c)


@pytest.fixture(name="mock_fixture_test_express_sim")
def fixture_mock_test_express_sim(requests_mock, express_sim):
    url = base_url + "/oauth/v1/generate?grant_type=client_credentials"
    requests_mock.get(url, json=AUTH)
    url = base_url + "/mpesa/stkpush/v1/processrequest"
    requests_mock.post(url, json=express_sim)


@pytest.fixture(name="mock_fixture_test_express_query")
def fixture_mock_test_express_query(requests_mock, express_query):
    url = base_url + "/oauth/v1/generate?grant_type=client_credentials"
    requests_mock.get(url, json=AUTH)
    url = base_url + "/mpesa/stkpushquery/v1/query"
    requests_mock.post(url, json=express_query)


@pytest.fixture(name="mock_fixture_test_c2b_reg")
def fixture_mock_test_c2b_reg(requests_mock, c2b_reg):
    url = base_url + "/oauth/v1/generate?grant_type=client_credentials"
    requests_mock.get(url, json=AUTH)
    url = base_url + "/mpesa/c2b/v1/registerurl"
    requests_mock.post(url, json=c2b_reg)


@pytest.fixture(name="mock_fixture_test_c2b_sim")
def fixture_mock_test_c2b_sim(requests_mock, c2b_sim):
    url = base_url + "/oauth/v1/generate?grant_type=client_credentials"
    requests_mock.get(url, json=AUTH)
    url = base_url + "/mpesa/c2b/v1/simulate"
    requests_mock.post(url, json=c2b_sim)


@pytest.fixture(name="mock_fixture_test_bal")
def fixture_mock_test_bal(requests_mock, bal):
    url = base_url + "/oauth/v1/generate?grant_type=client_credentials"
    requests_mock.get(url, json=AUTH)
    url = base_url + "/mpesa/accountbalance/v1/query"
    requests_mock.post(url, json=bal)


@pytest.fixture(name="mock_fixture_test_t_status")
def fixture_mock_test_t_status(requests_mock, t_status):
    url = base_url + "/oauth/v1/generate?grant_type=client_credentials"
    requests_mock.get(url, json=AUTH)
    url = base_url + "/mpesa/transactionstatus/v1/query"
    requests_mock.post(url, json=t_status)


@pytest.fixture(name="mock_fixture_test_b2b")
def fixture_mock_test_b2b(requests_mock, b2b):
    url = base_url + "/oauth/v1/generate?grant_type=client_credentials"
    requests_mock.get(url, json=AUTH)
    url = base_url + "/mpesa/b2b/v1/paymentrequest"
    requests_mock.post(url, json=b2b)



import requests
from .auth import MpesaBase


class Balance(MpesaBase):
    def __init__(self, env="sandbox", app_key=None, app_secret=None, sandbox_url="https://sandbox.safaricom.co.ke",
                 live_url="https://safaricom.co.ke"):
        MpesaBase.__init__(self, env, app_key, app_secret,
                           sandbox_url, live_url)
        self.authentication_token = self.authenticate()

    def get_balance(self, initiator=None, security_credential=None, command_id=None, party_a=None, identifier_type=None,
                    remarks=None, queue_timeout_url=None, result_url=None):
        """This method uses Mpesa's Account Balance API to to enquire the balance on an M-Pesa BuyGoods (Till Number).

        **Args:**
            - initiator (str): Username used to authenticate the transaction.
            - security_credential (str): Generate from developer portal.
            - command_id (str): AccountBalance.
            - party_a (int): Till number being queried.
            - identifier_type (int): Type of organization receiving the transaction.
             Options: 1 - MSISDN 2 - Till Number  4 - Organization short code
            - remarks (str): Comments that are sent along with the transaction(maximum 100 characters).
            - queue_timeout_url (str): The url that handles information of timed out transactions.
            - result_url (str): The url that receives results from M-Pesa api call.


        **Returns:**
            - OriginatorConverstionID (str): The unique request ID for tracking a transaction.
            - ConversationID (str): The unique request ID returned by mpesa for each request made
            - ResponseDescription (str): Response Description message


        """

        payload = {
            "Initiator": initiator,
            "SecurityCredential": security_credential,
            "CommandID": command_id,
            "PartyA": party_a,
            "IdentifierType": identifier_type,
            "Remarks": remarks,
            "QueueTimeOutURL": queue_timeout_url,
            "ResultURL": result_url
        }
        headers = {'Authorization': f"Bearer {self.authentication_token}", 'Content-Type': "application/json"}
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        saf_url = f"{base_safaricom_url}/mpesa/accountbalance/v1/query"
        r = requests.post(saf_url, headers=headers, json=payload)
        return r.json()



#!/usr/bin/env python
from setuptools import find_packages, setup

with open('README.md') as f:
    readme = f.read()

setup(
    name='Flask-Mpesa',
    version='1.2.7',
    url='https://github.com/allansifuna/flask-mpesa',
    license='BSD',
    author='Allan Namasaka Sifuna',
    author_email='allansifuna324@gmail.com',
    description='A Safaricom\'s DarajaAPI2.0 Package for Flask Applications.',
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Flask',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



import requests
from .auth import MpesaBase


class Reversal(MpesaBase):
    def __init__(self, env="sandbox", app_key=None, app_secret=None, sandbox_url=None, live_url=None):
        MpesaBase.__init__(self, env, app_key, app_secret,
                           sandbox_url, live_url)
        self.authentication_token = self.authenticate()

    def reverse(self, initiator=None, security_credential=None, command_id="TransactionReversal", transaction_id=None,
                amount=None, receiver_party=None, receiver_identifier_type=None, queue_timeout_url=None,
                result_url=None, remarks=None, occassion=None):
        """This method uses Mpesa's Transaction Reversal API to reverse a M-Pesa transaction.

        **Args:**
            - initiator (str): Username used to authenticate the transaction.
            - security_credential (str): Generate from developer portal
            - command_id (str): TransactionReversal
            - transaction_id (str): Unique identifier to identify a transaction on M-Pesa.
            - amount (int): The amount being transacted
            - receiver_party (int): Organization/MSISDN making the transaction - Shortcode (6 digits) - MSISDN (12 digits).
            - receiver_identifier_type (int): MSISDN receiving the transaction (12 digits).
            - queue_timeout_url (str): The url that handles information of timed out transactions.
            - result_url (str): The url that receives results from M-Pesa api call.
            - remarks (str): Comments that are sent along with the transaction(maximum 100 characters)
            - occassion (str):


        **Returns:**
            - OriginatorConverstionID (str): The unique request ID for tracking a transaction.
            - ConversationID (str): The unique request ID returned by mpesa for each request made
            - ResponseDescription (str): Response Description message


        """

        payload = {
            "Initiator": initiator,
            "SecurityCredential": security_credential,
            "CommandID": command_id,
            "TransactionID": transaction_id,
            "Amount": amount,
            "ReceiverParty": receiver_party,
            "ReceiverIdentifierType": receiver_identifier_type,
            "QueueTimeOutURL": queue_timeout_url,
            "ResultURL": result_url,
            "Remarks": remarks,
            "Occassion": occassion
        }
        headers = {'Authorization': f"Bearer {self.authentication_token}", 'Content-Type': "application/json"}
        if self.env == "production":
            base_safaricom_url = self.live_url
        else:
            base_safaricom_url = self.sandbox_url
        saf_url = f"{base_safaricom_url}/mpesa/reversal/v1/request"
        r = requests.post(saf_url, headers=headers, json=payload)
        return r.json()
